from django.urls import NoReverseMatch, reverse


class TestFunctionBaseMeta(type):
    def __new__(mcs, name, bases, attrs):
        def create_func(test_name, help_text, input_args, expected_value):
            def func(self):
                self._test_page_base(*input_args, result=expected_value)

            func.__name__ = f"test_{test_name}"
            func.__doc__ = help_text
            return func

        for test_name, help_text, input_args, expected_value in mcs.get_test_args(mcs.data_dict):
            func = create_func(test_name, help_text, input_args, expected_value)
            attrs[func.__name__] = func
        return type.__new__(mcs, name, bases, attrs)

    def _test_page_base(self, profile, action, url_name, args=None, data=None, result=None):
        if action == "length":
            # result: elements per page
            # FORBIDDEN
            # NOT FOUND
            try:
                endpoint = reverse(url_name)
                response = self.client.get(endpoint, format='json')
                self.assertEqual(len(response.json()['results']), result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "list":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.get(endpoint, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "retrieve":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.get(endpoint, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "create":
            # CREATED
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.post(endpoint, data, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "update":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.patch(endpoint, data, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "destroy":
            # NO_CONTENT
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.delete(endpoint, format='json')
                self.assertEqual(response.status_code, result)
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "status_set":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                data = {'status': 1}
                response = self.client.patch(endpoint, data, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "status_reset":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                data = {'status': 0}
                response = self.client.patch(endpoint, data, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "enable":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.patch(endpoint, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "disable":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.patch(endpoint, format='json')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if action == "fail":
            try:
                endpoint = reverse(url_name)
                reverse(endpoint)
                self.fail('endpoint to delete')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

    @classmethod
    def get_test_args(mcs, data_dict):
        for item in data_dict:
            yield (item['name'],
                   item['help'],
                   (item['profile'], item['action'], item['url_name']),
                   item['result'])


class TestFunctionMultipartMeta(type):
    """Variante di TestFunctionBaseMeta idonea per upload che avvengano con
       ``multipart/form-data``
    """

    def __new__(mcs, name, bases, attrs):
        def create_func(test_name, help_text, input_args, expected_value):
            def func(self):
                self._test_page_base(*input_args, result=expected_value)

            func.__name__ = f"test_{test_name}"
            func.__doc__ = help_text
            return func

        for test_name, help_text, input_args, expected_value in mcs.get_test_args(mcs.data_dict):
            func = create_func(test_name, help_text, input_args, expected_value)
            attrs[func.__name__] = func
        return type.__new__(mcs, name, bases, attrs)

    def _test_page_multipart(self, typology, url_name, args=None, result=None, data=None):
        if typology == "create":
            # CREATED
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.post(endpoint, data, content='multipart/form-data')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)

        if typology == "update":
            # SUCCESS
            # FORBIDDEN
            # NOT_FOUND
            try:
                endpoint = reverse(url_name, args=args)
                response = self.client.patch(endpoint, data, content='multipart/form-data')
                self.assertEqual(response.status_code, result)
                self.assertEqual(response['content-type'], 'application/json')
            except NoReverseMatch:
                self.assertTrue(result == self.NOT_FOUND)
